@@ -6,14 +6,17 @@
 
 import sys
 import ctypes
-
+import ctypes.util
 
 if sys.version_info[0] >= 3:
     BUFFER_TYPES = (memoryview, bytearray,)
+    BYTES_TYPE = bytes
 elif sys.version_info[1] >= 7:
     BUFFER_TYPES = (buffer, memoryview, bytearray,)  # noqa: F821
+    BYTES_TYPE = str
 else:
     BUFFER_TYPES = (buffer, bytearray,)  # noqa: F821
+    BYTES_TYPE = str
 
 
 # Find the base library and declare prototypes.
@@ -29,12 +32,29 @@ def _guess_lib_name():
     return ('libchromaprint.so.1', 'libchromaprint.so.0')
 
 
-for name in _guess_lib_name():
+def _load_library(name):
+    """Try to load a dynamic library with ctypes, or return None if the
+    library is not available.
+    """
+    if sys.platform == 'win32':
+        # On Windows since Python 3.8, we need an extra call to
+        # `find_library` to search standard library paths.
+        name = ctypes.util.find_library(name)
+        if not name:
+            return None
+
     try:
-        _libchromaprint = ctypes.cdll.LoadLibrary(name)
-        break
+        # Use `winmode=0` to search for libraries in extended search paths.
+        # https://learn.microsoft.com/en-us/windows/win32/api/libloaderapi/nf-libloaderapi-loadlibraryexa
+        return ctypes.CDLL(name, winmode=0)
     except OSError:
-        pass
+        return None
+
+
+for name in _guess_lib_name():
+    _libchromaprint = _load_library(name)
+    if _libchromaprint:
+        break
 else:
     raise ImportError("couldn't find libchromaprint")
 
@@ -65,7 +85,7 @@ _libchromaprint.chromaprint_get_fingerprint.restype = ctypes.c_int
 
 _libchromaprint.chromaprint_decode_fingerprint.argtypes = \
     (ctypes.POINTER(ctypes.c_char), ctypes.c_int,
-     ctypes.POINTER(ctypes.POINTER(ctypes.c_int32)),
+     ctypes.POINTER(ctypes.POINTER(ctypes.c_uint32)),
      ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int), ctypes.c_int)
 _libchromaprint.chromaprint_decode_fingerprint.restype = ctypes.c_int
 
@@ -75,6 +95,11 @@ _libchromaprint.chromaprint_encode_fingerprint.argtypes = \
      ctypes.POINTER(ctypes.c_int), ctypes.c_int)
 _libchromaprint.chromaprint_encode_fingerprint.restype = ctypes.c_int
 
+_libchromaprint.chromaprint_hash_fingerprint.argtypes = \
+    (ctypes.POINTER(ctypes.c_int32), ctypes.c_int,
+     ctypes.POINTER(ctypes.c_uint32))
+_libchromaprint.chromaprint_hash_fingerprint.restype = ctypes.c_int
+
 _libchromaprint.chromaprint_dealloc.argtypes = (ctypes.c_void_p,)
 _libchromaprint.chromaprint_dealloc.restype = None
 
@@ -119,7 +144,7 @@ class Fingerprinter(object):
         either a bytestring or a buffer object.
         """
         if isinstance(data, BUFFER_TYPES):
-            data = str(data)
+            data = BYTES_TYPE(data)
         elif not isinstance(data, bytes):
             raise TypeError('data must be bytes, buffer, or memoryview')
         _check(_libchromaprint.chromaprint_feed(
@@ -141,7 +166,18 @@ class Fingerprinter(object):
 
 
 def decode_fingerprint(data, base64=True):
-    result_ptr = ctypes.POINTER(ctypes.c_int32)()
+    """Uncompress and optionally decode a fingerprint.
+
+    Args:
+        data: An encoded fingerprint in bytes.
+        base64: Whether to base64-decode the fingerprint.
+
+    Returns:
+        A tuple containing the decoded raw fingerprint as an array
+        of unsigned 32-bit integers, and an int representing the chromaprint
+        algorithm used to generate the fingerprint.
+    """
+    result_ptr = ctypes.POINTER(ctypes.c_uint32)()
     result_size = ctypes.c_int()
     algorithm = ctypes.c_int()
     _check(_libchromaprint.chromaprint_decode_fingerprint(
@@ -154,6 +190,16 @@ def decode_fingerprint(data, base64=True):
 
 
 def encode_fingerprint(fingerprint, algorithm, base64=True):
+    """Compress and optionally encode a fingerprint.
+
+    Args:
+        fingerprint: A bytestring with the fingerprint.
+        algorithm: An int flag choosing the algorithm to use.
+        base64: Whether to base64-encode the fingerprint.
+
+    Returns:
+        A bytestring with the encoded fingerprint.
+    """
     fp_array = (ctypes.c_int * len(fingerprint))()
     for i in range(len(fingerprint)):
         fp_array[i] = fingerprint[i]
@@ -166,3 +212,46 @@ def encode_fingerprint(fingerprint, algorithm, base64=True):
     result = result_ptr[:result_size.value]
     _libchromaprint.chromaprint_dealloc(result_ptr)
     return result
+
+
+def hash_fingerprint(fingerprint):
+    """Generate a single 32-bit hash for a raw, decoded fingerprint.
+
+    If two fingerprints are similar, their hashes generated by this
+    function will also be similar. If they are significantly different,
+    their hashes will most likely be significantly different as well
+    (but clients should not rely on this).
+
+    Compare two hashes with their Hamming distance, i.e., by counting
+    the bits in which they differ.
+
+    Args:
+        fingerprint: A list of ints for the raw, decoded fingerprint.
+
+    Returns:
+        A 32-bit integer hash.
+
+    Example usage:
+        audio_fingerprint = <get fingerprint with Fingerprinter>
+        decoded_fingerprint, algo = decode_fingerprint(audio_fingerprint)
+        first_fingerprint_hash = hash_fingerprint(decoded_fingerprint)
+
+        second_fingerprint_hash = <repeat steps for second audio file>
+
+        # Compare the binary strings using Hamming distance.
+        first_fp_binary = format(first_fingerprint_hash, 'b')
+        second_fp_binary = format(second_fingerprint_hash, 'b')
+
+        # This value will be between 0 and 32 and represent the POPCNT.
+        # A value > 15 indicates the two fingerprints are very different.
+        bin(int(first_fp_binary,2)^int(second_fp_binary,2)).count
+    """
+
+    fp_array = (ctypes.c_int * len(fingerprint))()
+    for i in range(len(fingerprint)):
+        fp_array[i] = fingerprint[i]
+    result_hash = ctypes.c_uint32()
+    _check(_libchromaprint.chromaprint_hash_fingerprint(
+        fp_array, len(fingerprint), ctypes.byref(result_hash)
+    ))
+    return result_hash.value
